/*
 * Copyright 2011 Steven Gribble
 *
 *  This file is part of the UW CSE 333 course project sequence
 *  (333proj).
 *
 *  333proj is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  333proj is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with 333proj.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <string.h>

#include "doctable.h"
#include "libhw1/hashtable.h"

// This is the structure that represents a DocTable; it
// contains two hash tables, one mapping from docid to
// docname, and one mapping from docname to docid.
typedef struct DocTableRecord {
  HashTable docid_to_docname;  // mapping docid to docname
  HashTable docname_to_docid;  // mapping docname to docid
  uint64_t max_id;             // max docID allocated so far
} DocTableRecord, *DocTableRecordPtr;

DocTable AllocateDocTable(void) {
  DocTableRecord *dt = (DocTableRecord *) malloc(sizeof(DocTableRecord));
  assert(dt != NULL);

  dt->docid_to_docname = AllocateHashTable(1024);
  dt->docname_to_docid = AllocateHashTable(1024);
  dt->max_id = 0;

  assert(dt->docid_to_docname != NULL);
  assert(dt->docname_to_docid != NULL);
  return (DocTable) dt;
}

void FreeDocTable(DocTable table) {
  DocTableRecord *dt = (DocTableRecord *) table;
  assert(dt != NULL);

  // STEP 1

}

uint64_t DTNumDocsInDocTable(DocTable table) {
  DocTableRecord *dt = (DocTableRecord *) table;
  assert(dt != NULL);

  return NumElementsInHashTable(dt->docid_to_docname);
}

uint64_t DTRegisterDocumentName(DocTable table, char *docname) {
  // Allocate space for the docid and a copy of the docname.
  DocTableRecord *dt = (DocTableRecord *) table;
  char       *doccopy = strdup(docname);
  uint64_t   *docid = (uint64_t *) malloc(sizeof(uint64_t));
  uint64_t   res;
  HTKeyValue kv, oldkv;

  assert(dt != NULL);
  assert(doccopy != NULL);
  assert(docid != NULL);

  // Check to see if the document already exists; if so,
  // free up the malloc'ed space and return the existing docid

  // STEP 2


  // allocate the next docID
  dt->max_id += 1;
  *docid = dt->max_id;

  // Set up the key/value for the docid_to_docname mapping, and
  // do the insert.

  // STEP 3


  // Set up the key/value for the docname_to_docid mapping, and
  // do the insert.

  // STEP 4


  return *docid;
}

uint64_t DTLookupDocumentName(DocTable table, char *docname) {
  DocTableRecord *dt = (DocTableRecord *) table;
  uint64_t key;
  HTKeyValue kv;
  int res;

  assert(dt != NULL);
  assert(docname != NULL);

  // Lookup the FNVHash64() or the docname in the
  // docname_to_docid table within dt, and return
  // either "0" if the docname isn't found or the
  // docID if it is.

  // STEP 5

}

char *DTLookupDocID(DocTable table, uint64_t docid) {
  DocTableRecord *dt = (DocTableRecord *) table;
  HTKeyValue kv;
  int res;

  assert(dt != NULL);
  assert(docid != 0);

  // Lookup the docid in the docid_to_docname table,
  // and either return the string (i.e., the (char *)
  // saved in the value field for that key) or
  // NULL if the key isn't in the table.

  // STEP 6


}

HashTable DTGetDocidTable(DocTable table) {
  DocTableRecord *dt = (DocTableRecord *) table;

  assert(dt != NULL);
  return dt->docid_to_docname;
}
