/*
 * Copyright 2011 Steven Gribble
 *
 *  This file is part of the UW CSE 333 course project sequence
 *  (333proj).
 *
 *  333proj is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  333proj is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with 333proj.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <string.h>
#include <unistd.h>
#include <assert.h>
#include <ctype.h>

#include "memindex.h"
#include "filecrawler.h"

static void Usage(void);
static void ProcessQueries(DocTable dt, MemIndex mi);
static char *GetNextLine(FILE *f);

int main(int argc, char **argv) {
  int res;
  DocTable dt;
  MemIndex mi;

  if (argc != 2)
    Usage();

  // Crawl and index the file tree provided to us by the user.
  printf("Indexing '%s'\n", argv[1]);
  res = CrawlFileTree(argv[1], &dt, &mi);
  if (res == 0) {
    fprintf(stderr, "Path '%s' is not indexable\n", argv[1]);
    Usage();
  }

  // Process queries.
  ProcessQueries(dt, mi);

  // Should never return from ProcessQueries, but if we
  // do, clean up shop before quitting.
  FreeDocTable(dt);
  FreeMemIndex(mi);
  return EXIT_SUCCESS;
}

static void Usage(void) {
  fprintf(stderr, "Usage: ./searchshell <docroot>\n");
  fprintf(stderr,
          "where <docroot> is an absolute or relative " \
          "path to a directory to build an index under.\n");
  exit(-1);
}

static void ProcessQueries(DocTable dt, MemIndex mi) {
  // Loop forever, receiving a query from the user,
  // splitting it up into words, and passing it to
  // the memindex.c/h MIProcessQuery() function.
  while (1) {
    char *lineptr = NULL;

    fprintf(stdout, "enter query:\n");

    // Use our private GetNextLine() function to read the next line of
    // input from stdin.  STEP 1 is to implement the GetNextLine()
    // function,  which is at the bottom of this file.
    lineptr = GetNextLine(stdin);
    assert(lineptr != NULL);

    // Make sure the user passed us a non-empty search
    // string.  If so, parse it.
    if (strlen(lineptr) > 0) {
      int wordcount = 0, i;
      char *wordarray[255];  // hardcode a maximum of 255 search words.
      LinkedList qres;

      // Write a loop that uses the "strtok()" stdio string function
      // to extract each query word from the query string.
      // You should assume that query words are separated by space
      // characters (i.e., " " is a good 'delim' argument
      // to pass strtok()).   ("man strtok")

      // STEP 2


      // For each word that you found, convert that word
      // to lower-case by using the string.h "tolower"
      // macro.  ("man tolower")

      // STEP 3



      // Process the query!
      if (wordcount > 0) {
        // Invoke the "MIProcessQuery()" function in
        // memindex.h/c to process the query against
        // the inverted index.  If the query succeeds,
        // then interate through the query result list,
        // printing it out with printf.  Try to mimic the
        // format that we demonstrated in the HW2 web page
        // (i.e., mimic the transcript that is linked
        // off of it).

        // STEP 4


      }
    } else {
      // We read an empty line, so exit the searchshell!
      free(lineptr);
      break;
    }

    // Free the line that getline() returned to us.
    free(lineptr);
  }
}

// This utility function reads the next line from file F
// into a C string in heap-allocated memory and returns a pointer
// to it.  A line is defined to be zero or more characters that
// end at a either a newline character or the end of the file.
//
// If the line ends with a newline character ('\n'), the
// newline character is discarded (overwritten with '\0').  Therefore,
// this function will return an empty string (just the '\0')
// if it encounters an empty line in the file.
//
// On error, returns NULL.  Caller is responsible for
// fclose()'ing the file; this function never does that.
//
// If the file is empty, i.e., GetNextLine() hits EOF before
// reading any characters, it returns an empty string (i.e.,
// "").
static char *GetNextLine(FILE *f) {
  char *retbuf = NULL;
  int numread = 0;
  int done = 0;

  // STEP 1 -- implement GetNextLine().  I recommend using
  // fgets() and realloc() to do it.  (fgets() to read some
  // number of characters per loop iteration, and realloc
  // to grow the buffer you're reading into on each loop
  // iteration.)  You can use whatever loop construct you
  // like; I like using while (!done) { .. }, but you can
  // do something else if it makes sense to do so.
  while (!done) {


  }
  return retbuf;
}
