/*
 * Copyright 2011 Steven Gribble
 *
 *  This file is part of the UW CSE 333 course project sequence
 *  (333proj).
 *
 *  333proj is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  333proj is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with 333proj.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <assert.h>

#include "CUnit/Basic.h"
#include "fileparser.h"

extern void MISAddPoints(int points);

void FPTestReadFile(void) {
  char *filebuf = NULL;
  uint64_t len = 0;

  // read a big file
  filebuf = ReadFile("./test_tree/books/paradiselost.txt", &len);
  CU_ASSERT_PTR_NOT_NULL_FATAL(filebuf);
  CU_ASSERT_EQUAL_FATAL(len, 495687);
  CU_ASSERT_EQUAL_FATAL(filebuf[0], 'T');
  CU_ASSERT_EQUAL_FATAL(filebuf[495686], ((char) 0x0a));
  free(filebuf);
  MISAddPoints(10);

  // try to read a non-existent file
  filebuf = ReadFile("bogus/file", &len);
  CU_ASSERT_PTR_NULL_FATAL(filebuf);
  MISAddPoints(10);

  // read a small file
  filebuf = ReadFile("./test_tree/bash-4.2/trap.h", &len);
  CU_ASSERT_PTR_NOT_NULL_FATAL(filebuf);
  CU_ASSERT_EQUAL_FATAL(len, 3213);
  CU_ASSERT_EQUAL_FATAL(filebuf[0], '/');
  CU_ASSERT_EQUAL_FATAL(filebuf[3212], ((char) 0x0a));
  free(filebuf);
  MISAddPoints(10);

  // try to read a file that contains non-ASCII text
  filebuf = ReadFile("./test_tree./libhw1.a", &len);
  CU_ASSERT_PTR_NULL_FATAL(filebuf);
  MISAddPoints(10);
}

void FPTestBuildWordHT(void) {
  HashTable tab = BuildWordHT("./test_tree/bash-4.2/doc/README");
  CU_ASSERT_PTR_NOT_NULL_FATAL(tab);
  CU_ASSERT_EQUAL_FATAL(NumElementsInHashTable(tab), 91);
  MISAddPoints(10);

  // Do a few spot checks.
  {
    // article: 154, 170; identical: 918; versions: 499, 550, 653.
    char *w1 = "article", *w2 = "identical", *w3 = "versions";
    HTKeyValue kv;
    int retval;
    WordPositions *wp;
    uint32_t pos;

    retval = LookupHashTable(tab,
                             FNVHash64((unsigned char *) w1, strlen(w1)),
                             &kv);
    CU_ASSERT_EQUAL_FATAL(retval, 1);
    wp = (WordPositions *) kv.value;
    CU_ASSERT_EQUAL_FATAL(strcmp(w1, wp->word), 0);
    CU_ASSERT_EQUAL_FATAL(NumElementsInLinkedList(wp->positions), 2);
    retval = PopLinkedList(wp->positions, (void **) &pos);
    CU_ASSERT_EQUAL_FATAL(retval, 1);
    CU_ASSERT_EQUAL_FATAL(pos, 154);
    retval = PopLinkedList(wp->positions, (void **) &pos);
    CU_ASSERT_EQUAL_FATAL(retval, 1);
    CU_ASSERT_EQUAL_FATAL(pos, 170);
    MISAddPoints(10);

    retval = LookupHashTable(tab,
                             FNVHash64((unsigned char *) w2, strlen(w2)),
                             &kv);
    CU_ASSERT_EQUAL_FATAL(retval, 1);
    wp = (WordPositions *) kv.value;
    CU_ASSERT_EQUAL_FATAL(strcmp(w2, wp->word), 0);
    CU_ASSERT_EQUAL_FATAL(NumElementsInLinkedList(wp->positions), 1);
    retval = PopLinkedList(wp->positions, (void **) &pos);
    CU_ASSERT_EQUAL_FATAL(retval, 1);
    CU_ASSERT_EQUAL_FATAL(pos, 918);
    MISAddPoints(10);

    retval = LookupHashTable(tab,
                             FNVHash64((unsigned char *) w3, strlen(w3)),
                             &kv);
    CU_ASSERT_EQUAL_FATAL(retval, 1);
    wp = (WordPositions *) kv.value;
    CU_ASSERT_EQUAL_FATAL(strcmp(w3, wp->word), 0);
    CU_ASSERT_EQUAL_FATAL(NumElementsInLinkedList(wp->positions), 3);
    retval = PopLinkedList(wp->positions, (void **) &pos);
    CU_ASSERT_EQUAL_FATAL(retval, 1);
    CU_ASSERT_EQUAL_FATAL(pos, 499);
    retval = PopLinkedList(wp->positions, (void **) &pos);
    CU_ASSERT_EQUAL_FATAL(retval, 1);
    CU_ASSERT_EQUAL_FATAL(pos, 550);
    retval = PopLinkedList(wp->positions, (void **) &pos);
    CU_ASSERT_EQUAL_FATAL(retval, 1);
    CU_ASSERT_EQUAL_FATAL(pos, 653);
    MISAddPoints(10);

    retval = LookupHashTable(tab,
                             FNVHash64((unsigned char *) "blah", 4),
                             &kv);
    CU_ASSERT_EQUAL_FATAL(retval, 0);
    MISAddPoints(10);
  }

  FreeWordHT(tab);
  MISAddPoints(10);
}

void FPTestBigBuildWordHT(void) {
  HashTable tab = BuildWordHT("./test_tree/books/paradiselost.txt");
  CU_ASSERT_PTR_NOT_NULL_FATAL(tab);
  CU_ASSERT_EQUAL_FATAL(NumElementsInHashTable(tab), 10123);
  FreeWordHT(tab);
  MISAddPoints(10);
}
