/*
 * Copyright 2011 Steven Gribble
 *
 *  This file is part of the UW CSE 333 course project sequence
 *  (333proj).
 *
 *  333proj is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  333proj is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with 333proj.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <assert.h>

#include "CUnit/Basic.h"
#include "memindex.h"
#include "filecrawler.h"

extern void MISAddPoints(int points);

void MITestSimple(void) {
  int res;
  DocTable dt;
  MemIndex idx;
  LinkedList llres;
  LLIter lit;
  int i;

  char *q1[] = {"equations"};
  char *q2[] = {"report", "normal"};
  char *q3[] = {"report", "suggestions", "normal"};
  char *q4[] = {"report", "normal", "foobarbaz"};

  // Crawl part of the file tree.
  res = CrawlFileTree("./test_tree/bash-4.2/support", &dt, &idx);
  CU_ASSERT_EQUAL_FATAL(res, 1);
  CU_ASSERT_EQUAL_FATAL(MINumWordsInMemIndex(idx), 3852);

  // Process query 1, check results.
  llres = MIProcessQuery(idx, q1, 1);
  CU_ASSERT_PTR_NOT_NULL_FATAL(llres);
  CU_ASSERT_EQUAL_FATAL(NumElementsInLinkedList(llres), 2);
  lit = LinkedListMakeIterator(llres, 0);
  for (i = 0; i < NumElementsInLinkedList(llres); i++) {
    SearchResult *res;

    CU_ASSERT_EQUAL_FATAL(LLIteratorGetPayload(lit, (void **) &res), 1);
    if (i == 0) {
      char *docname = DTLookupDocID(dt, res->docid);
      CU_ASSERT_EQUAL_FATAL(
         strcmp(docname,
                "./test_tree/bash-4.2/support/texi2html"), 0);
    } else if (i == 1) {
      char *docname = DTLookupDocID(dt, res->docid);
      CU_ASSERT_EQUAL_FATAL(
         strcmp(docname,
                "./test_tree/bash-4.2/support/man2html.c"), 0);
    }
    LLIteratorNext(lit);
  }
  LLIteratorFree(lit);
  FreeLinkedList(llres, &free);
  MISAddPoints(20);

  // Process query 2, check results.
  llres = MIProcessQuery(idx, q2, 2);
  CU_ASSERT_PTR_NOT_NULL_FATAL(llres);
  CU_ASSERT_EQUAL_FATAL(NumElementsInLinkedList(llres), 2);
  lit = LinkedListMakeIterator(llres, 0);
  for (i = 0; i < NumElementsInLinkedList(llres); i++) {
    SearchResult *res;

    CU_ASSERT_EQUAL_FATAL(LLIteratorGetPayload(lit, (void **) &res), 1);
    if (i == 0) {
      char *docname = DTLookupDocID(dt, res->docid);
      CU_ASSERT_EQUAL_FATAL(
         strcmp(docname,
                "./test_tree/bash-4.2/support/texi2html"), 0);
    } else if (i == 1) {
      char *docname = DTLookupDocID(dt, res->docid);
      CU_ASSERT_EQUAL_FATAL(
         strcmp(docname,
                "./test_tree/bash-4.2/support/man2html.c"), 0);
    }
    LLIteratorNext(lit);
  }
  LLIteratorFree(lit);
  FreeLinkedList(llres, &free);
  MISAddPoints(20);

  // Process query 3, check results.
  llres = MIProcessQuery(idx, q3, 3);
  CU_ASSERT_PTR_NOT_NULL_FATAL(llres);
  CU_ASSERT_EQUAL_FATAL(NumElementsInLinkedList(llres), 1);
  lit = LinkedListMakeIterator(llres, 0);
  for (i = 0; i < NumElementsInLinkedList(llres); i++) {
    SearchResult *res;

    CU_ASSERT_EQUAL_FATAL(LLIteratorGetPayload(lit, (void **) &res), 1);
    if (i == 0) {
      char *docname = DTLookupDocID(dt, res->docid);
      CU_ASSERT_EQUAL_FATAL(
         strcmp(docname,
                "./test_tree/bash-4.2/support/texi2html"), 0);
    }
    LLIteratorNext(lit);
  }
  LLIteratorFree(lit);
  FreeLinkedList(llres, &free);
  MISAddPoints(20);

  // Process query 4, check results.
  llres = MIProcessQuery(idx, q4, 3);
  CU_ASSERT_PTR_NULL_FATAL(llres);
  MISAddPoints(20);

  // Free up everything.
  FreeDocTable(dt);
  FreeMemIndex(idx);
  MISAddPoints(10);
}
