/*
 * Copyright 2011 Steven Gribble
 *
 *  This file is part of the UW CSE 333 course project sequence
 *  (333proj).
 *
 *  333proj is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  333proj is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with 333proj.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <assert.h>

#include "CUnit/Basic.h"

// our CUnit suite initialization/cleanup functions
void MISAddPoints(int points);
int  MISInit(void);
int  MISCleanup(void);

// our CUnit test functions

// fileparser.c tests
extern void FPTestReadFile(void);
extern void FPTestBuildWordHT(void);
extern void FPTestBigBuildWordHT(void);

// doctable.c tests
extern void DTTestSimple(void);

// memindex.c tests
extern void MITestSimple(void);

// filecrawler.c tests
extern void FCTestSimple(void);

// our main function
int main(int argc, char **argv) {
  CU_pSuite pSuite = NULL;

  // initialize the test registry
  if (CU_initialize_registry() != CUE_SUCCESS) {
    return CU_get_error();
  }

  // add a suite to the registry
  pSuite = CU_add_suite("hash table suite", MISInit, MISCleanup);

  if (pSuite == NULL) {
    CU_cleanup_registry();
    return CU_get_error();
  }

  // add the tests to the suite
  if ((CU_ADD_TEST(pSuite, FPTestReadFile) == NULL) ||
      (CU_ADD_TEST(pSuite, FPTestBuildWordHT) == NULL) ||
      (CU_ADD_TEST(pSuite, FPTestBigBuildWordHT) == NULL) ||
      (CU_ADD_TEST(pSuite, DTTestSimple) == NULL) ||
      (CU_ADD_TEST(pSuite, FCTestSimple) == NULL) ||
      (CU_ADD_TEST(pSuite, MITestSimple) == NULL)) {
    CU_cleanup_registry();
    return CU_get_error();
  }

  // run all of the tests using the CUnit basic interface
  CU_basic_set_mode(CU_BRM_VERBOSE);
  CU_basic_run_tests();
  CU_cleanup_registry();
  return CU_get_error();
}


static const int MAXGRADE = 270;
static int yourgrade = 0;

void MISAddPoints(int points) {
  yourgrade += points;
  printf("(%d/%d)", yourgrade, MAXGRADE);
}

void MISFinalGrade(void) {
  printf("\n\nYour unit test grade is %d/%d [%.02f%%]\n",
         yourgrade, MAXGRADE, (100.0*yourgrade)/MAXGRADE);
}

int MISInit(void) {
  yourgrade = 0;
  printf("There are %d unit-testing points available.\n",
         MAXGRADE);
  return 0;  // return 0 on success, non-zero otherwise
}

int MISCleanup(void) {
  MISFinalGrade();
  return 0;  // return 0 on success, non-zero otherwise
}
